

// Initialization of the ADC
void setupADC()
{

  // clear ADLAR in ADMUX (0x7C) to right-adjust the result
  // ADCL will contain lower 8 bits, ADCH upper 2 (in last two bits)
  ADMUX &= B11011111;

  // Set REFS1..0 in ADMUX (0x7C) to change reference voltage to the
  // proper source (01)
  ADMUX |= B01000000;

  // Clear MUX3..0 in ADMUX (0x7C) in preparation for setting the analog
  // input
  ADMUX &= B11110000;

  // Set MUX3..0 in ADMUX (0x7C) to read intially from AD0
  ADMUX |= B00000000; //set to A0

  // Set ADEN in ADCSRA (0x7A) to enable the ADC.
  // Note, this instruction takes 12 ADC clocks to execute
  ADCSRA |= B10000000;

  // Set ADATE in ADCSRA (0x7A) auto-triggering.
  //ADCSRA |= B00100000; // enable auto-triggering
  ADCSRA |= B00000000; //disable auto

  // Clear ADTS2..0 in ADCSRB (0x7B) to set trigger mode to free running.
  // This means that as soon as an ADC has finished, the next will be
  // immediately started.
  ADCSRB &= B11111000;

  // Set the Prescaler to 128 (16000KHz/128 = 125KHz)
  // Above 200KHz 10-bit results are not reliable.
  ADCSRA &= B11111000; // clear divider
  //ADCSRA |= B00000111; // divide by 128
  ADCSRA |= B00000100; // divide by 16

  // USING INTERRUPTS
  //Set ADIE in ADCSRA (0x7A) to enable the ADC interrupt.
  // Without this, the internal interrupt will not trigger.
  ADCSRA |= B00001000;

  // USING INTERRUPTS
  // Enable global interrupts
  // AVR macro included in <avr/interrupts.h>, which the Arduino IDE
  // supplies by default.
  sei();

  // Kick off the first ADC
  // Set ADSC in ADCSRA (0x7A) to start the ADC conversion
  ADCSRA |= B01000000;
}
//------------------------------------------------------------------

// Interrupt service routine for the ADC completion
ISR(ADC_vect) {

  //  monitor ADC speed on pin A4
  /*   HEARTBEATB = HEARTBEATB ^ 1;
    if (HEARTBEATB == 0) {
      PORTC = PORTC & B11101111;
    }
    else
    {
      PORTC = PORTC | B00010000;
    }
  */

  if (analogSource == 0)  // clock in
  {
    analogSource = 1;
    ADMUX ++;
    avalCL = ADCL | (ADCH << 8);
  }
  else if (analogSource == 1) // reset in
  {
    ADMUX ++;
    analogSource = 2;
    avalRST = ADCL | (ADCH << 8);

  }
  else if (analogSource == 2) // encoder switches
  {
    ADMUX &= B11110000;
    analogSource = 0;
    avalSW = ADCL | (ADCH << 8);

  }

  // Set ADSC in ADCSRA (0x7A) to start another ADC conversion
  ADCSRA |= B01000000;
}
